from ipaddress import IPv4Interface, IPv4Network
from subprocess import PIPE, run
from typing import List, Tuple, Dict
import sys

from .WireGuardConfiguration import ServerConf, ClientConf


def is_valid_network(ip: str) -> bool:
    """check if network is in valid cidr notation"""
    try:
        IPv4Network(ip)
    except ValueError:
        return False
    return True


def has_enough_addresses(ip: str, num_clients: int) -> bool:
    """check if network has enough addreses for number of clients
    account for network + broadcast addresses + server address"""
    if num_clients > IPv4Network(ip).num_addresses - 3:
        return False
    return True


def exit_if_no_wg_tools() -> None:
    try:
        run(["wg"])
    except FileNotFoundError:
        sys.exit("You might not have wireguard-tools intalled. Exiting...")


def gen_key_pair() -> Dict[str, str]:
    """generate and return a wireguard key pair"""
    wg_genkey_output = run(["wg", "genkey"], stdout=PIPE)
    wg_pubkey_output = run(
        ["wg", "pubkey"], input=wg_genkey_output.stdout, capture_output=True
    )
    private_key = wg_genkey_output.stdout.decode().strip()
    public_key = wg_pubkey_output.stdout.decode().strip()
    return {"private_key": private_key, "public_key": public_key}


def gen_client_keys(num_clients: int) -> Dict[int, Dict[str, str]]:
    """generate client key pairs and stores them in a dictionary"""
    client_keys_dict = {i: gen_key_pair() for i in range(1, num_clients + 1)}
    return client_keys_dict


def create_confs(
    server_private_key: str,
    server_public_key: str,
    client_keys_dict: Dict[int, Dict[str, str]],
    network_address: IPv4Interface,
    endpoint: str,
    port: int,
    allowed_ips: str,
) -> Tuple[ServerConf, List[ClientConf]]:
    """create server and client configs"""
    server_ip = network_address + 1
    # initialize client_ip
    client_ip = server_ip

    server_peers_list: List[str] = []
    client_confs: List[ClientConf] = []

    for k in client_keys_dict:
        client_ip += 1
        server_peers_list.append(
            f"[Peer]\n"
            f'PublicKey = {client_keys_dict[k]["public_key"]}\n'
            f"AllowedIPs = {client_ip}\n\n"
        )

        client_confs.append(
            ClientConf(
                client_keys_dict[k]["private_key"],
                client_ip,
                server_public_key,
                allowed_ips,
                endpoint,
                port,
            )
        )

    server_peers = "".join(server_peers_list)

    server_conf = ServerConf(server_private_key, server_ip, port, server_peers)

    return server_conf, client_confs



from abc import ABC, abstractmethod
from ipaddress import IPv4Interface
from pathlib import Path
from shutil import rmtree
import qrcode as qr
import ipaddress

CONFIG_PATH = Path(".") / "configs"


class WireGuardConfiguration(ABC):
    def __init__(self, private_key: str, ip_address: IPv4Interface):
        self.private_key = private_key
        self.ip_address = ip_address
        self.path = CONFIG_PATH

        if self.path.exists():
            rmtree(self.path)
        self.path.mkdir()

    @abstractmethod
    def make_conf(self):
        """abstract method implemented in subclasses"""
        return

    def generate_qr_code(self, filename: str) -> None:
        """generate qr code from configuration file"""
        with open(self.path / f"{filename}.conf", "r") as input_data:
            img = qr.make(input_data.read())
            img.save(self.path / f"{filename}.png")

    def write_conf_to_file(self, filename: str, conf: str) -> None:
        """write configuration to file"""
        with open(self.path / filename, "w") as f:
            f.write(conf.strip())


class ServerConf(WireGuardConfiguration):
    def __init__(
        self,
        private_key: str,
        ip_address: ipaddress.IPv4Interface,
        port: int,
        peers: str,
    ):

        super().__init__(private_key, ip_address)
        self.port = port
        self.peers = peers

    def make_conf(self) -> str:
        return (
            f"[Interface]\n"
            f"Address = {self.ip_address}\n"
            f"ListenPort = {self.port}\n"
            f"PrivateKey = {self.private_key}\n\n"
            f"{self.peers}"
        )


class ClientConf(WireGuardConfiguration):
    def __init__(
        self,
        private_key: str,
        ip_address: ipaddress.IPv4Interface,
        server_public_key: str,
        allowed_ips: ipaddress.IPv4Network,
        endpoint: str,
        port: int,
    ):

        super().__init__(private_key, ip_address)
        self.server_public_key = server_public_key
        self.allowed_ips = allowed_ips
        self.endpoint = endpoint
        self.port = port

    def make_conf(self) -> str:
        return (
            f"[Interface]\n"
            f"Address = {self.ip_address}\n"
            f"PrivateKey = {self.private_key}\n\n"
            f"[Peer]\n"
            f"PublicKey = {self.server_public_key}\n"
            f"AllowedIPs = {self.allowed_ips}\n"
            f"Endpoint = {self.endpoint}:{self.port}\n\n"
        )



from InquirerPy import inquirer
from InquirerPy.validator import NumberValidator
from ipaddress import IPv4Interface
from .utils import (
    exit_if_no_wg_tools,
    gen_key_pair,
    gen_client_keys,
    create_confs,
    is_valid_network,
    has_enough_addresses,
)


def main():
    exit_if_no_wg_tools()

    num_clients = inquirer.text(
        message="Enter number of devices:",
        validate=NumberValidator(),
        filter=lambda result: int(result),
    ).execute()

    def network_validator(result) -> bool:
        """Ensure the input is not empty."""
        return is_valid_network(result) and has_enough_addresses(result, num_clients)

    network = inquirer.text(
        message="Enter a network in CIDR Notation",
        validate=network_validator,
        filter=lambda result: IPv4Interface(result),
        invalid_message="Either invalid CIDR or not enough address space. The default of 10.0.0.0/24 should work for most configurations.",
        default="10.0.0.0/24",
    ).execute()

    endpoint = inquirer.text(
        message="Enter an endpoint for your Wireguard server (e.g. an IP address or hostname):",
    ).execute()

    port = inquirer.text(
        message="Enter a port for your endpoint. Press Enter for default:",
        validate=NumberValidator(),
        filter=lambda result: int(result),
        default="51820",
    ).execute()

    allowed_ips = inquirer.text(
        message="Which IP addresses are allowed to communicate with your client?",
        default=str(network),
    ).execute()

    server_key_pair = gen_key_pair()
    client_keys_dict = gen_client_keys(num_clients)

    server_conf, client_confs = create_confs(
        server_key_pair["private_key"],
        server_key_pair["public_key"],
        client_keys_dict,
        network,
        endpoint,
        port,
        allowed_ips,
    )

    server_conf.write_conf_to_file("01_server.conf", server_conf.make_conf())
    if num_clients <= 20:
        server_conf.generate_qr_code("01_server")

    for i, conf in enumerate(client_confs, start=1):
        conf.write_conf_to_file(f"{i + 1:02d}_client_{i}.conf", conf.make_conf())
        conf.generate_qr_code(f"{i+1:02d}_client_{i}")

    print(
        f"\nWireguard Server: {conf.endpoint}:{server_conf.port}\n"
        f"{i} clients\n"
        "Configuration files and QR codes stored in ./configs"
    )


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['wgconf']

package_data = \
{'': ['*']}

install_requires = \
['Pillow==8.3.1',
 'inquirerpy==0.3.0',
 'pfzy==0.3.3',
 'prompt-toolkit==3.0.22',
 'qrcode==7.2',
 'wcwidth==0.2.5']

entry_points = \
{'console_scripts': ['wgconf = wgconf.prompt:main']}

setup_kwargs = {
    'name': 'wgconf',
    'version': '0.3.0',
    'description': 'Interactive Wireguard configuration generator',
    'long_description': None,
    'author': 'Andy Ta',
    'author_email': 'doktorfaustus@live.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
